@@ -46,6 +46,14 @@
 #include "gdm-session-worker.h"
 #include "gdm-marshal.h"
 
+#if defined (HAVE_ADT)
+#include "gdm-session-solaris-auditor.h"
+#elif defined (HAVE_LIBAUDIT)
+#include "gdm-session-linux-auditor.h"
+#else
+#include "gdm-session-auditor.h"
+#endif
+
 #define GDM_SESSION_WORKER_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GDM_TYPE_SESSION_WORKER, GdmSessionWorkerPrivate))
 
 #define GDM_SESSION_DBUS_PATH         "/org/gnome/DisplayManager/Session"
@@ -94,6 +102,7 @@ struct GdmSessionWorkerPrivate
         /* from Setup */
         char             *service;
         char             *x11_display_name;
+        char             *x11_authority_file;
         char             *display_device;
         char             *hostname;
         char             *username;
@@ -109,6 +118,8 @@ struct GdmSessionWorkerPrivate
 
         char             *server_address;
         DBusConnection   *connection;
+
+        GdmSessionAuditor *auditor;
 };
 
 enum {
@@ -484,6 +495,8 @@ gdm_session_worker_update_username (GdmSessionWorker *worker)
                          username != NULL ? username : "<unset>",
                          worker->priv->username != NULL ? worker->priv->username : "<unset>");
 
+                gdm_session_auditor_set_username (worker->priv->auditor, worker->priv->username);
+
                 if ((worker->priv->username == username) ||
                     ((worker->priv->username != NULL) && (username != NULL) &&
                      (strcmp (worker->priv->username, username) == 0)))
@@ -771,6 +784,32 @@ gdm_session_worker_pam_new_messages_handler (int                        number_o
         return return_value;
 }
 
+static void
+gdm_session_worker_start_auditor (GdmSessionWorker *worker)
+{
+
+/* FIXME: it may make sense at some point to keep a list of
+ * auditors, instead of assuming they are mutually exclusive
+ */
+#if defined (HAVE_ADT)
+        worker->priv->auditor = gdm_session_solaris_auditor_new (worker->priv->hostname,
+                                                                 worker->priv->display_device);
+#elif defined (HAVE_LIBAUDIT)
+        worker->priv->auditor = gdm_session_linux_auditor_new (worker->priv->hostname,
+                                                               worker->priv->display_device);
+#else
+        worker->priv->auditor = gdm_session_auditor_new (worker->priv->hostname,
+                                                         worker->priv->display_device);
+#endif
+}
+
+static void
+gdm_session_worker_stop_auditor (GdmSessionWorker *worker)
+{
+        g_object_unref (worker->priv->auditor);
+        worker->priv->auditor = NULL;
+}
+
 static void
 gdm_session_worker_uninitialize_pam (GdmSessionWorker *worker,
                                      int               status)
@@ -786,11 +825,18 @@ gdm_session_worker_uninitialize_pam (GdmSessionWorker *worker,
 
         if (worker->priv->state >= GDM_SESSION_WORKER_STATE_SESSION_OPENED) {
                 pam_close_session (worker->priv->pam_handle, 0);
+                gdm_session_auditor_report_logout (worker->priv->auditor);
+        } else {
+                gdm_session_auditor_report_login_failure (worker->priv->auditor,
+                                                          status,
+                                                          pam_strerror (worker->priv->pam_handle, status));
         }
 
         pam_end (worker->priv->pam_handle, status);
         worker->priv->pam_handle = NULL;
 
+        gdm_session_worker_stop_auditor (worker);
+
         g_debug ("GdmSessionWorker: state NONE");
         worker->priv->state = GDM_SESSION_WORKER_STATE_NONE;
 }
@@ -801,8 +847,9 @@ _get_tty_for_pam (const char *x11_display_name,
 {
 #ifdef __sun
         return g_strdup (display_device);
-#endif
+#else
         return g_strdup (x11_display_name);
+#endif
 }
 
 static gboolean
@@ -811,6 +858,7 @@ gdm_session_worker_initialize_pam (GdmSessionWorker *worker,
                                    const char       *username,
                                    const char       *hostname,
                                    const char       *x11_display_name,
+                                   const char       *x11_authority_file,
                                    const char       *display_device,
                                    GError          **error)
 {
@@ -825,13 +873,14 @@ gdm_session_worker_initialize_pam (GdmSessionWorker *worker,
         pam_conversation.conv = (GdmSessionWorkerPamNewMessagesFunc) gdm_session_worker_pam_new_messages_handler;
         pam_conversation.appdata_ptr = worker;
 
+        gdm_session_worker_start_auditor (worker);
         error_code = pam_start (service,
                                 username,
                                 &pam_conversation,
                                 &worker->priv->pam_handle);
 
         if (error_code != PAM_SUCCESS) {
-                g_debug ("GdmSessionWorker: could not initialize pam");
+                g_debug ("GdmSessionWorker: could not initialize PAM");
                 /* we don't use pam_strerror here because it requires a valid
                  * pam handle, and if pam_start fails pam_handle is undefined
                  */
@@ -842,7 +891,7 @@ gdm_session_worker_initialize_pam (GdmSessionWorker *worker,
                              error_code == PAM_ABORT? _("general failure") :
                              error_code == PAM_BUF_ERR? _("out of memory") :
                              error_code == PAM_SYSTEM_ERR? _("application programmer error") :
-                             _("unscoped error"));
+                             _("unknown error"));
 
                 goto out;
         }
@@ -964,9 +1013,16 @@ gdm_session_worker_authorize_user (GdmSessionWorker *worker,
 
         /* it's possible that the user needs to change their password or pin code
          */
-        if (error_code == PAM_NEW_AUTHTOK_REQD)
+        if (error_code == PAM_NEW_AUTHTOK_REQD) {
                 error_code = pam_chauthtok (worker->priv->pam_handle, PAM_CHANGE_EXPIRED_AUTHTOK);
 
+                if (error_code != PAM_SUCCESS) {
+                        gdm_session_auditor_report_password_change_failure (worker->priv->auditor);
+                } else {
+                        gdm_session_auditor_report_password_changed (worker->priv->auditor);
+                }
+        }
+
         if (error_code != PAM_SUCCESS) {
                 g_debug ("GdmSessionWorker: user is not authorized to log in: %s",
                          pam_strerror (worker->priv->pam_handle, error_code));
@@ -1142,6 +1198,8 @@ gdm_session_worker_accredit_user (GdmSessionWorker  *worker,
                 goto out;
         }
 
+        gdm_session_auditor_report_user_accredited (worker->priv->auditor);
+
         g_debug ("GdmSessionWorker: state ACCREDITED");
         worker->priv->state = GDM_SESSION_WORKER_STATE_ACCREDITED;
 
@@ -1291,8 +1349,7 @@ _open_session_log (const char *dir)
                         goto out;
                 }
 
-                g_warning (_("session log '%s' is not a normal file, "
-                             "logging session to '%s' instead.\n"), filename,
+                g_warning ("session log '%s' is not a normal file, logging session to '%s' instead.\n", filename,
                            temp_name);
                 g_free (filename);
                 filename = temp_name;
@@ -1315,7 +1372,7 @@ out:
         g_free (filename);
 
         if (fd < 0) {
-                g_warning (_("unable to log session"));
+                g_warning ("unable to log session");
                 fd = g_open ("/dev/null", O_RDWR);
         }
 
@@ -1444,6 +1501,8 @@ gdm_session_worker_open_user_session (GdmSessionWorker  *worker,
                 return FALSE;
         }
 
+        gdm_session_auditor_report_login (worker->priv->auditor);
+
         return TRUE;
 }
 
@@ -1531,6 +1590,7 @@ do_setup (GdmSessionWorker *worker)
                                                  worker->priv->username,
                                                  worker->priv->hostname,
                                                  worker->priv->x11_display_name,
+                                                 worker->priv->x11_authority_file,
                                                  worker->priv->display_device,
                                                  &error);
         if (! res) {
@@ -1788,6 +1848,7 @@ on_setup (GdmSessionWorker *worker,
         DBusError   error;
         const char *service;
         const char *x11_display_name;
+        const char *x11_authority_file;
         const char *console;
         const char *hostname;
         dbus_bool_t res;
@@ -1801,10 +1862,12 @@ on_setup (GdmSessionWorker *worker,
                                      DBUS_TYPE_STRING, &x11_display_name,
                                      DBUS_TYPE_STRING, &console,
                                      DBUS_TYPE_STRING, &hostname,
+                                     DBUS_TYPE_STRING, &x11_authority_file,
                                      DBUS_TYPE_INVALID);
         if (res) {
                 worker->priv->service = g_strdup (service);
                 worker->priv->x11_display_name = g_strdup (x11_display_name);
+                worker->priv->x11_authority_file = g_strdup (x11_authority_file);
                 worker->priv->display_device = g_strdup (console);
                 worker->priv->hostname = g_strdup (hostname);
                 worker->priv->username = NULL;
@@ -1824,6 +1887,7 @@ on_setup_for_user (GdmSessionWorker *worker,
         DBusError   error;
         const char *service;
         const char *x11_display_name;
+        const char *x11_authority_file;
         const char *console;
         const char *hostname;
         const char *username;
@@ -1838,11 +1902,13 @@ on_setup_for_user (GdmSessionWorker *worker,
                                      DBUS_TYPE_STRING, &x11_display_name,
                                      DBUS_TYPE_STRING, &console,
                                      DBUS_TYPE_STRING, &hostname,
+                                     DBUS_TYPE_STRING, &x11_authority_file,
                                      DBUS_TYPE_STRING, &username,
                                      DBUS_TYPE_INVALID);
         if (res) {
                 worker->priv->service = g_strdup (service);
                 worker->priv->x11_display_name = g_strdup (x11_display_name);
+                worker->priv->x11_authority_file = g_strdup (x11_authority_file);
                 worker->priv->display_device = g_strdup (console);
                 worker->priv->hostname = g_strdup (hostname);
                 worker->priv->username = g_strdup (username);
