@@ -31,6 +31,12 @@
 #include <sys/socket.h>
 #include <sys/time.h>
 #include <errno.h>
+#include <pwd.h>
+
+#ifdef ENABLE_RBAC_SHUTDOWN
+#include <auth_attr.h>
+#include <secdb.h>
+#endif
 
 #include <glib.h>
 #include <glib/gi18n.h>
@@ -44,9 +50,16 @@
 
 #include <glade/glade-xml.h>
 
+#define DBUS_API_SUBJECT_TO_CHANGE
+#include <dbus/dbus-glib.h>
+#include <dbus/dbus-glib-lowlevel.h>
+
+#ifdef HAVE_POLKIT_GNOME
+#include <polkit-gnome/polkit-gnome.h>
+#endif
+
 #include "gdm-greeter-login-window.h"
 #include "gdm-user-chooser-widget.h"
-#include "gdm-session-chooser-widget.h"
 
 #if HAVE_PAM
 #include <security/pam_appl.h>
@@ -55,6 +68,15 @@
 #define PW_ENTRY_SIZE GDM_MAX_PASS
 #endif
 
+#define CK_NAME      "org.freedesktop.ConsoleKit"
+#define CK_PATH      "/org/freedesktop/ConsoleKit"
+#define CK_INTERFACE "org.freedesktop.ConsoleKit"
+
+#define CK_MANAGER_PATH      "/org/freedesktop/ConsoleKit/Manager"
+#define CK_MANAGER_INTERFACE "org.freedesktop.ConsoleKit.Manager"
+#define CK_SEAT_INTERFACE    "org.freedesktop.ConsoleKit.Seat"
+#define CK_SESSION_INTERFACE "org.freedesktop.ConsoleKit.Session"
+
 #define GLADE_XML_FILE "gdm-greeter-login-window.glade"
 
 #define GDM_GREETER_LOGIN_WINDOW_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GDM_TYPE_GREETER_LOGIN_WINDOW, GdmGreeterLoginWindowPrivate))
@@ -67,12 +89,9 @@ enum {
 struct GdmGreeterLoginWindowPrivate
 {
         GladeXML        *xml;
-        GtkWidget       *session_chooser;
         GtkWidget       *user_chooser;
         gboolean         display_is_local;
         char            *timeformat;
-        guint            update_clock_id;
-        gboolean         clock_show_seconds;
 
         guint            animation_timeout_id;
 };
@@ -86,10 +105,7 @@ enum {
         BEGIN_VERIFICATION,
         BEGIN_VERIFICATION_FOR_USER,
         QUERY_ANSWER,
-        SESSION_SELECTED,
-        LANGUAGE_SELECTED,
         USER_SELECTED,
-        HOSTNAME_SELECTED,
         DISCONNECTED,
         CANCELLED,
         LAST_SIGNAL
@@ -101,8 +117,6 @@ static void     gdm_greeter_login_window_class_init   (GdmGreeterLoginWindowClas
 static void     gdm_greeter_login_window_init         (GdmGreeterLoginWindow      *greeter_login_window);
 static void     gdm_greeter_login_window_finalize     (GObject                    *object);
 
-static gboolean update_clock_timeout_cb               (GdmGreeterLoginWindow *login_window);
-
 G_DEFINE_TYPE (GdmGreeterLoginWindow, gdm_greeter_login_window, GTK_TYPE_WINDOW)
 
 static void
@@ -180,6 +194,31 @@ show_widget (GdmGreeterLoginWindow *login_window,
         }
 }
 
+static gboolean
+get_show_restart_buttons (GdmGreeterLoginWindow *login_window)
+{
+        gboolean show;
+
+        show = TRUE;
+
+#ifdef ENABLE_RBAC_SHUTDOWN
+        {
+                char *username;
+
+                username = g_get_user_name ();
+                if (username == NULL || !chkauthattr (RBAC_SHUTDOWN_KEY, username)) {
+                        show = FALSE;
+                        g_debug ("Not showing stop/restart buttons for user %s due to RBAC key %s",
+                                 username, RBAC_SHUTDOWN_KEY);
+                } else {
+                        g_debug ("Showing stop/restart buttons for user %s due to RBAC key %s",
+                                 username, RBAC_SHUTDOWN_KEY);
+                }
+        }
+#endif
+        return show;
+}
+
 static void
 switch_mode (GdmGreeterLoginWindow *login_window,
              int                    number)
@@ -187,20 +226,24 @@ switch_mode (GdmGreeterLoginWindow *login_window,
         const char *default_name;
         GtkWidget  *user_chooser;
         GtkWidget  *box;
+        gboolean    show_restart_shutdown = TRUE;
 
         /* FIXME: do animation */
         default_name = NULL;
 
+        show_restart_shutdown = get_show_restart_buttons (login_window);
+
         switch (number) {
         case MODE_SELECTION:
                 show_widget (login_window, "log-in-button", FALSE);
                 show_widget (login_window, "cancel-button", FALSE);
-                show_widget (login_window, "shutdown-button", login_window->priv->display_is_local);
-                show_widget (login_window, "restart-button", login_window->priv->display_is_local);
+                show_widget (login_window, "shutdown-button",
+                             login_window->priv->display_is_local && show_restart_shutdown);
+                show_widget (login_window, "restart-button",
+                             login_window->priv->display_is_local && show_restart_shutdown);
                 show_widget (login_window, "suspend-button", login_window->priv->display_is_local);
                 show_widget (login_window, "disconnect-button", ! login_window->priv->display_is_local);
                 show_widget (login_window, "auth-input-box", FALSE);
-                show_widget (login_window, "session-chooser", FALSE);
                 default_name = NULL;
                 break;
         case MODE_AUTHENTICATION:
@@ -210,8 +253,6 @@ switch_mode (GdmGreeterLoginWindow *login_window,
                 show_widget (login_window, "restart-button", FALSE);
                 show_widget (login_window, "suspend-button", FALSE);
                 show_widget (login_window, "disconnect-button", FALSE);
-                show_widget (login_window, "auth-input-box", TRUE);
-                show_widget (login_window, "session-chooser", TRUE);
                 default_name = "log-in-button";
                 break;
         default:
@@ -255,8 +296,6 @@ do_cancel (GdmGreeterLoginWindow *login_window)
 {
 
         gdm_user_chooser_widget_set_chosen_user_name (GDM_USER_CHOOSER_WIDGET (login_window->priv->user_chooser), NULL);
-        gdm_session_chooser_widget_set_current_session_name (GDM_SESSION_CHOOSER_WIDGET (login_window->priv->session_chooser),
-                                                             GDM_SESSION_CHOOSER_SESSION_PREVIOUS);
 
         switch_mode (login_window, MODE_SELECTION);
         set_busy (login_window);
@@ -285,8 +324,6 @@ reset_dialog (GdmGreeterLoginWindow *login_window)
         set_message (login_window, "");
 
         gdm_user_chooser_widget_set_chosen_user_name (GDM_USER_CHOOSER_WIDGET (login_window->priv->user_chooser), NULL);
-        gdm_session_chooser_widget_set_current_session_name (GDM_SESSION_CHOOSER_WIDGET (login_window->priv->session_chooser),
-                                                             GDM_SESSION_CHOOSER_SESSION_PREVIOUS);
 
         switch_mode (login_window, MODE_SELECTION);
 
@@ -361,6 +398,7 @@ gdm_greeter_login_window_info_query (GdmGreeterLoginWindow *login_window,
         label = glade_xml_get_widget (GDM_GREETER_LOGIN_WINDOW (login_window)->priv->xml, "auth-prompt-label");
         gtk_label_set_text (GTK_LABEL (label), text);
 
+        show_widget (login_window, "auth-input-box", TRUE);
         set_sensitive (GDM_GREETER_LOGIN_WINDOW (login_window), TRUE);
         set_ready (GDM_GREETER_LOGIN_WINDOW (login_window));
         set_focus (GDM_GREETER_LOGIN_WINDOW (login_window));
@@ -384,6 +422,7 @@ gdm_greeter_login_window_secret_info_query (GdmGreeterLoginWindow *login_window,
         label = glade_xml_get_widget (GDM_GREETER_LOGIN_WINDOW (login_window)->priv->xml, "auth-prompt-label");
         gtk_label_set_text (GTK_LABEL (label), text);
 
+        show_widget (login_window, "auth-input-box", TRUE);
         set_sensitive (GDM_GREETER_LOGIN_WINDOW (login_window), TRUE);
         set_ready (GDM_GREETER_LOGIN_WINDOW (login_window));
         set_focus (GDM_GREETER_LOGIN_WINDOW (login_window));
@@ -461,6 +500,265 @@ cancel_button_clicked (GtkButton             *button,
         do_cancel (login_window);
 }
 
+static gboolean
+try_system_stop (DBusGConnection *connection,
+                 GError         **error)
+{
+        DBusGProxy      *proxy;
+        gboolean         res;
+
+        g_debug ("GdmGreeterLoginWindow: trying to stop system");
+
+        proxy = dbus_g_proxy_new_for_name (connection,
+                                           CK_NAME,
+                                           CK_MANAGER_PATH,
+                                           CK_MANAGER_INTERFACE);
+        res = dbus_g_proxy_call_with_timeout (proxy,
+                                              "Stop",
+                                              INT_MAX,
+                                              error,
+                                              /* parameters: */
+                                              G_TYPE_INVALID,
+                                              /* return values: */
+                                              G_TYPE_INVALID);
+        return res;
+}
+
+static gboolean
+try_system_restart (DBusGConnection *connection,
+                    GError         **error)
+{
+        DBusGProxy      *proxy;
+        gboolean         res;
+
+        g_debug ("GdmGreeterLoginWindow: trying to restart system");
+
+        proxy = dbus_g_proxy_new_for_name (connection,
+                                           CK_NAME,
+                                           CK_MANAGER_PATH,
+                                           CK_MANAGER_INTERFACE);
+        res = dbus_g_proxy_call_with_timeout (proxy,
+                                              "Restart",
+                                              INT_MAX,
+                                              error,
+                                              /* parameters: */
+                                              G_TYPE_INVALID,
+                                              /* return values: */
+                                              G_TYPE_INVALID);
+        return res;
+}
+
+#ifdef HAVE_POLKIT_GNOME
+static void
+system_restart_auth_cb (PolKitAction          *action,
+                        gboolean               gained_privilege,
+                        GError                *error,
+                        GdmGreeterLoginWindow *login_window)
+{
+        GError          *local_error;
+        DBusGConnection *connection;
+        gboolean         res;
+
+        g_debug ("GdmGreeterLoginWindow: system restart auth callback gained=%s", gained_privilege ? "yes" : "no");
+
+        if (! gained_privilege) {
+                if (error != NULL) {
+                        g_warning ("GdmGreeterLoginWindow: system restart error: %s", error->message);
+                }
+                return;
+        }
+
+        local_error = NULL;
+        connection = dbus_g_bus_get (DBUS_BUS_SYSTEM, &local_error);
+        if (connection == NULL) {
+                g_warning ("Unable to get system bus connection: %s", local_error->message);
+                g_error_free (local_error);
+                return;
+        }
+
+        res = try_system_restart (connection, &local_error);
+        if (! res) {
+                g_warning ("Unable to restart system: %s", local_error->message);
+                g_error_free (local_error);
+                return;
+        }
+}
+
+static void
+system_stop_auth_cb (PolKitAction          *action,
+                     gboolean               gained_privilege,
+                     GError                *error,
+                     GdmGreeterLoginWindow *login_window)
+{
+        GError          *local_error;
+        DBusGConnection *connection;
+        gboolean         res;
+
+        g_debug ("GdmGreeterLoginWindow: system stop auth callback gained=%s", gained_privilege ? "yes" : "no");
+
+        if (! gained_privilege) {
+                if (error != NULL) {
+                        g_warning ("GdmGreeterLoginWindow: system stop error: %s", error->message);
+                }
+                return;
+        }
+
+        local_error = NULL;
+        connection = dbus_g_bus_get (DBUS_BUS_SYSTEM, &local_error);
+        if (connection == NULL) {
+                g_warning ("Unable to get system bus connection: %s", local_error->message);
+                g_error_free (local_error);
+                return;
+        }
+
+        res = try_system_stop (connection, &local_error);
+        if (! res) {
+                g_warning ("Unable to stop system: %s", local_error->message);
+                g_error_free (local_error);
+                return;
+        }
+}
+
+static PolKitAction *
+get_action_from_error (GError *error)
+{
+        PolKitAction *action;
+        const char   *paction;
+
+        action = polkit_action_new ();
+
+        paction = NULL;
+        if (g_str_has_prefix (error->message, "Not privileged for action: ")) {
+                paction = error->message + strlen ("Not privileged for action: ");
+        }
+        g_debug ("Requesting priv for '%s'", paction);
+
+        polkit_action_set_action_id (action, paction);
+
+        return action;
+}
+#endif
+
+static void
+do_system_restart (GdmGreeterLoginWindow *login_window)
+{
+        gboolean         res;
+        GError          *error;
+        DBusGConnection *connection;
+
+        error = NULL;
+        connection = dbus_g_bus_get (DBUS_BUS_SYSTEM, &error);
+        if (connection == NULL) {
+                g_warning ("Unable to get system bus connection: %s", error->message);
+                g_error_free (error);
+                return;
+        }
+
+        res = try_system_restart (connection, &error);
+#ifdef HAVE_POLKIT_GNOME
+        if (! res) {
+                g_debug ("GdmGreeterLoginWindow: unable to restart system: %s: %s",
+                         dbus_g_error_get_name (error),
+                         error->message);
+
+                if (dbus_g_error_has_name (error, "org.freedesktop.ConsoleKit.Manager.NotPrivileged")) {
+                        PolKitAction *action;
+                        guint         xid;
+                        pid_t         pid;
+
+                        action = get_action_from_error (error);
+
+                        xid = 0;
+                        pid = getpid ();
+
+                        g_error_free (error);
+                        error = NULL;
+                        res = polkit_gnome_auth_obtain (action,
+                                                        xid,
+                                                        pid,
+                                                        (PolKitGnomeAuthCB) system_restart_auth_cb,
+                                                        login_window,
+                                                        &error);
+                        polkit_action_unref (action);
+
+                        if (! res) {
+                                g_warning ("Unable to request privilege for action: %s", error->message);
+                                g_error_free (error);
+                        }
+
+                }
+        }
+#endif
+}
+
+static void
+do_system_stop (GdmGreeterLoginWindow *login_window)
+{
+        gboolean         res;
+        GError          *error;
+        DBusGConnection *connection;
+
+        error = NULL;
+        connection = dbus_g_bus_get (DBUS_BUS_SYSTEM, &error);
+        if (connection == NULL) {
+                g_warning ("Unable to get system bus connection: %s", error->message);
+                g_error_free (error);
+                return;
+        }
+
+        res = try_system_stop (connection, &error);
+#ifdef HAVE_POLKIT_GNOME
+        if (! res) {
+                g_debug ("GdmGreeterLoginWindow: unable to stop system: %s: %s",
+                         dbus_g_error_get_name (error),
+                         error->message);
+
+                if (dbus_g_error_has_name (error, "org.freedesktop.ConsoleKit.Manager.NotPrivileged")) {
+                        PolKitAction *action;
+                        guint         xid;
+                        pid_t         pid;
+
+                        xid = 0;
+                        pid = getpid ();
+
+                        action = get_action_from_error (error);
+
+                        g_error_free (error);
+                        error = NULL;
+                        res = polkit_gnome_auth_obtain (action,
+                                                        xid,
+                                                        pid,
+                                                        (PolKitGnomeAuthCB) system_stop_auth_cb,
+                                                        login_window,
+                                                        &error);
+                        polkit_action_unref (action);
+
+                        if (! res) {
+                                g_warning ("Unable to request privilege for action: %s", error->message);
+                                g_error_free (error);
+                        }
+
+                }
+        }
+#endif
+}
+
+static void
+restart_button_clicked (GtkButton             *button,
+                        GdmGreeterLoginWindow *login_window)
+{
+        g_debug ("GdmGreeterLoginWindow: restart button clicked");
+        do_system_restart (login_window);
+}
+
+static void
+shutdown_button_clicked (GtkButton             *button,
+                         GdmGreeterLoginWindow *login_window)
+{
+        g_debug ("GdmGreeterLoginWindow: stop button clicked");
+        do_system_stop (login_window);
+}
+
 static void
 on_user_chosen (GdmUserChooserWidget  *user_chooser,
                 GdmGreeterLoginWindow *login_window)
@@ -472,6 +770,9 @@ on_user_chosen (GdmUserChooserWidget  *user_chooser,
                 return;
         }
 
+        g_signal_emit (G_OBJECT (login_window), signals[USER_SELECTED],
+                       0, user_name);
+
         if (strcmp (user_name, GDM_USER_CHOOSER_USER_OTHER) == 0) {
                 g_signal_emit (login_window, signals[BEGIN_VERIFICATION], 0);
         } else if (strcmp (user_name, GDM_USER_CHOOSER_USER_GUEST) == 0) {
@@ -487,104 +788,11 @@ on_user_chosen (GdmUserChooserWidget  *user_chooser,
 
 static void
 on_user_unchosen (GdmUserChooserWidget  *user_chooser,
-                     GdmGreeterLoginWindow *login_window)
+                  GdmGreeterLoginWindow *login_window)
 {
         do_cancel (login_window);
 }
 
-static void
-on_session_activated (GdmSessionChooserWidget *session_chooser,
-                      GdmGreeterLoginWindow   *login_window)
-{
-        char *session;
-
-        session = gdm_session_chooser_widget_get_current_session_name (GDM_SESSION_CHOOSER_WIDGET (login_window->priv->session_chooser));
-        if (session == NULL) {
-                return;
-        }
-
-        g_signal_emit (login_window, signals[SESSION_SELECTED], 0, session);
-
-        g_free (session);
-}
-
-static void
-update_clock (GtkLabel   *label,
-              const char *format)
-{
-        time_t     t;
-        struct tm *tm;
-        char       buf[256];
-        char      *utf8;
-
-        time (&t);
-        tm = localtime (&t);
-        if (tm == NULL) {
-                g_warning ("Unable to get broken down local time");
-                return;
-        }
-        if (strftime (buf, sizeof (buf), format, tm) == 0) {
-                g_warning ("Couldn't format time: %s", format);
-                strcpy (buf, "???");
-        }
-        utf8 = g_locale_to_utf8 (buf, -1, NULL, NULL, NULL);
-        gtk_label_set_text (label, utf8);
-        g_free (utf8);
-}
-
-static void
-set_clock_timeout (GdmGreeterLoginWindow *login_window,
-                   time_t                 now)
-{
-        struct timeval tv;
-        int            timeouttime;
-
-        if (login_window->priv->update_clock_id > 0) {
-                g_source_remove (login_window->priv->update_clock_id);
-                login_window->priv->update_clock_id = 0;
-        }
-
-        gettimeofday (&tv, NULL);
-        timeouttime = (G_USEC_PER_SEC - tv.tv_usec) / 1000 + 1;
-
-        /* timeout of one minute if we don't care about the seconds */
-        if (! login_window->priv->clock_show_seconds) {
-                timeouttime += 1000 * (59 - now % 60);
-        }
-
-        login_window->priv->update_clock_id = g_timeout_add (timeouttime,
-                                                             (GSourceFunc)update_clock_timeout_cb,
-                                                             login_window);
-
-}
-
-static gboolean
-update_clock_timeout_cb (GdmGreeterLoginWindow *login_window)
-{
-        GtkWidget *label;
-        time_t     new_time;
-
-        time (&new_time);
-
-        label = glade_xml_get_widget (login_window->priv->xml, "computer-info-time-label");
-        if (label != NULL) {
-                update_clock (GTK_LABEL (label), login_window->priv->timeformat);
-        }
-
-        set_clock_timeout (login_window, new_time);
-
-        return FALSE;
-}
-
-static void
-remove_clock_timeout (GdmGreeterLoginWindow *login_window)
-{
-        if (login_window->priv->update_clock_id > 0) {
-                g_source_remove (login_window->priv->update_clock_id);
-                login_window->priv->update_clock_id = 0;
-        }
-}
-
 static gboolean
 on_computer_info_label_button_press (GtkWidget             *widget,
                                      GdkEventButton        *event,
@@ -593,7 +801,6 @@ on_computer_info_label_button_press (GtkWidget             *widget,
         GtkWidget *notebook;
         int        current_page;
         int        n_pages;
-        GtkWidget *label;
 
         /* switch page */
         notebook = glade_xml_get_widget (login_window->priv->xml, "computer-info-notebook");
@@ -606,16 +813,6 @@ on_computer_info_label_button_press (GtkWidget             *widget,
                 gtk_notebook_set_current_page (GTK_NOTEBOOK (notebook), 0);
         }
 
-        /* if the clock is visible then start it */
-        label = glade_xml_get_widget (login_window->priv->xml, "computer-info-time-label");
-        if (gtk_notebook_get_current_page (GTK_NOTEBOOK (notebook)) == gtk_notebook_page_num (GTK_NOTEBOOK (notebook), label)) {
-                time_t now;
-                time (&now);
-                set_clock_timeout (login_window, now);
-        } else {
-                remove_clock_timeout (login_window);
-        }
-
         return FALSE;
 }
 
@@ -696,35 +893,6 @@ get_system_version (void)
         return version;
 }
 
-static char *
-get_time_format (GdmGreeterLoginWindow *login_window)
-{
-        const char *time_format;
-        const char *date_format;
-        char       *clock_format;
-        char       *result;
-
-        time_format = login_window->priv->clock_show_seconds ? _("%l:%M:%S %p") : _("%l:%M %p");
-        /* translators: replace %e with %d if, when the day of the
-         *              month as a decimal number is a single digit, it
-         *              should begin with a 0 in your locale (e.g. "May
-         *              01" instead of "May  1").
-         */
-        date_format = _("%a %b %e");
-        /* translators: reverse the order of these arguments
-         *              if the time should come before the
-         *              date on a clock in your locale.
-         */
-        clock_format = g_strdup_printf (_("%1$s, %2$s"),
-                                        date_format,
-                                        time_format);
-
-        result = g_locale_from_utf8 (clock_format, -1, NULL, NULL, NULL);
-        g_free (clock_format);
-
-        return result;
-}
-
 static void
 create_computer_info (GdmGreeterLoginWindow *login_window)
 {
@@ -742,13 +910,6 @@ create_computer_info (GdmGreeterLoginWindow *login_window)
                 gtk_label_set_text (GTK_LABEL (label), version);
                 g_free (version);
         }
-
-
-        label = glade_xml_get_widget (login_window->priv->xml, "computer-info-time-label");
-        if (label != NULL) {
-                login_window->priv->timeformat = get_time_format (login_window);
-                update_clock (GTK_LABEL (label), login_window->priv->timeformat);
-        }
 }
 
 #define INVISIBLE_CHAR_DEFAULT       '*'
@@ -777,8 +938,6 @@ custom_widget_constructor (GladeXML              *xml,
 
         if (strcmp (name, "user-chooser") == 0) {
                widget = gdm_user_chooser_widget_new ();
-        } else if (strcmp (name, "session-chooser") == 0) {
-               widget = gdm_session_chooser_widget_new ();
         }
 
         return widget;
@@ -826,24 +985,6 @@ load_theme (GdmGreeterLoginWindow *login_window)
 
         gtk_widget_show (login_window->priv->user_chooser);
 
-        login_window->priv->session_chooser =
-                glade_xml_get_widget (login_window->priv->xml, "session-chooser");
-
-        if (login_window->priv->session_chooser == NULL) {
-                g_critical ("Session chooser not found in greeter theme");
-        }
-
-        gdm_session_chooser_widget_set_show_only_chosen (GDM_SESSION_CHOOSER_WIDGET (login_window->priv->session_chooser), TRUE);
-
-        g_signal_connect (login_window->priv->session_chooser,
-                          "activated",
-                          G_CALLBACK (on_session_activated),
-                          login_window);
-
-        gdm_session_chooser_widget_set_current_session_name (GDM_SESSION_CHOOSER_WIDGET (login_window->priv->session_chooser),
-                                                             GDM_SESSION_CHOOSER_SESSION_PREVIOUS);
-
-
         button = glade_xml_get_widget (login_window->priv->xml, "log-in-button");
         gtk_widget_grab_default (button);
         g_signal_connect (button, "clicked", G_CALLBACK (log_in_button_clicked), login_window);
@@ -851,6 +992,11 @@ load_theme (GdmGreeterLoginWindow *login_window)
         button = glade_xml_get_widget (login_window->priv->xml, "cancel-button");
         g_signal_connect (button, "clicked", G_CALLBACK (cancel_button_clicked), login_window);
 
+        button = glade_xml_get_widget (login_window->priv->xml, "restart-button");
+        g_signal_connect (button, "clicked", G_CALLBACK (restart_button_clicked), login_window);
+        button = glade_xml_get_widget (login_window->priv->xml, "shutdown-button");
+        g_signal_connect (button, "clicked", G_CALLBACK (shutdown_button_clicked), login_window);
+
         entry = glade_xml_get_widget (login_window->priv->xml, "auth-prompt-entry");
         /* Only change the invisible character if it '*' otherwise assume it is OK */
         if ('*' == gtk_entry_get_invisible_char (GTK_ENTRY (entry))) {
@@ -1040,16 +1186,6 @@ gdm_greeter_login_window_class_init (GdmGreeterLoginWindowClass *klass)
                               g_cclosure_marshal_VOID__STRING,
                               G_TYPE_NONE,
                               1, G_TYPE_STRING);
-        signals [LANGUAGE_SELECTED] =
-                g_signal_new ("language-selected",
-                              G_TYPE_FROM_CLASS (object_class),
-                              G_SIGNAL_RUN_LAST,
-                              G_STRUCT_OFFSET (GdmGreeterLoginWindowClass, language_selected),
-                              NULL,
-                              NULL,
-                              g_cclosure_marshal_VOID__STRING,
-                              G_TYPE_NONE,
-                              1, G_TYPE_STRING);
         signals [USER_SELECTED] =
                 g_signal_new ("user-selected",
                               G_TYPE_FROM_CLASS (object_class),
@@ -1060,26 +1196,6 @@ gdm_greeter_login_window_class_init (GdmGreeterLoginWindowClass *klass)
                               g_cclosure_marshal_VOID__STRING,
                               G_TYPE_NONE,
                               1, G_TYPE_STRING);
-        signals [SESSION_SELECTED] =
-                g_signal_new ("session-selected",
-                              G_TYPE_FROM_CLASS (object_class),
-                              G_SIGNAL_RUN_LAST,
-                              G_STRUCT_OFFSET (GdmGreeterLoginWindowClass, session_selected),
-                              NULL,
-                              NULL,
-                              g_cclosure_marshal_VOID__STRING,
-                              G_TYPE_NONE,
-                              1, G_TYPE_STRING);
-        signals [HOSTNAME_SELECTED] =
-                g_signal_new ("hostname-selected",
-                              G_TYPE_FROM_CLASS (object_class),
-                              G_SIGNAL_RUN_LAST,
-                              G_STRUCT_OFFSET (GdmGreeterLoginWindowClass, hostname_selected),
-                              NULL,
-                              NULL,
-                              g_cclosure_marshal_VOID__STRING,
-                              G_TYPE_NONE,
-                              1, G_TYPE_STRING);
         signals [CANCELLED] =
                 g_signal_new ("cancelled",
                               G_TYPE_FROM_CLASS (object_class),
@@ -1116,8 +1232,7 @@ gdm_greeter_login_window_init (GdmGreeterLoginWindow *login_window)
 {
         login_window->priv = GDM_GREETER_LOGIN_WINDOW_GET_PRIVATE (login_window);
 
-        login_window->priv->clock_show_seconds = TRUE;
-
+        gtk_window_set_title (GTK_WINDOW (login_window), _("Login Window"));
         gtk_window_set_opacity (GTK_WINDOW (login_window), 0.85);
         gtk_window_set_position (GTK_WINDOW (login_window), GTK_WIN_POS_CENTER_ALWAYS);
         gtk_window_set_deletable (GTK_WINDOW (login_window), FALSE);
@@ -1140,8 +1255,6 @@ gdm_greeter_login_window_finalize (GObject *object)
 
         g_return_if_fail (login_window->priv != NULL);
 
-        remove_clock_timeout (login_window);
-
         G_OBJECT_CLASS (gdm_greeter_login_window_parent_class)->finalize (object);
 }
 
