@@ -28,11 +28,11 @@
 #include <glib.h>
 #include <glib/gi18n.h>
 #include <glib-object.h>
+#include <gconf/gconf-client.h>
 
 #include "gdm-greeter-session.h"
 #include "gdm-greeter-client.h"
 #include "gdm-greeter-panel.h"
-#include "gdm-greeter-background.h"
 #include "gdm-greeter-login-window.h"
 
 #include "gdm-session-manager.h"
@@ -40,14 +40,22 @@
 
 #define GDM_GREETER_SESSION_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GDM_TYPE_GREETER_SESSION, GdmGreeterSessionPrivate))
 
+#define KEY_GDM_A11Y_DIR            "/apps/gdm/simple-greeter/accessibility"
+#define KEY_SCREEN_KEYBOARD_ENABLED  KEY_GDM_A11Y_DIR "/screen_keyboard_enabled"
+#define KEY_SCREEN_MAGNIFIER_ENABLED KEY_GDM_A11Y_DIR "/screen_magnifier_enabled"
+#define KEY_SCREEN_READER_ENABLED    KEY_GDM_A11Y_DIR "/screen_reader_enabled"
+
 struct GdmGreeterSessionPrivate
 {
         GdmGreeterClient      *client;
         GdmSessionManager     *manager;
 
+        GdmSessionClient      *screen_reader_client;
+        GdmSessionClient      *screen_keyboard_client;
+        GdmSessionClient      *screen_magnifier_client;
+
         GtkWidget             *login_window;
         GtkWidget             *panel;
-        GtkWidget             *background;
 };
 
 enum {
@@ -97,6 +105,7 @@ on_reset (GdmGreeterClient  *client,
 {
         g_debug ("GdmGreeterSession: Reset");
 
+        gdm_greeter_panel_hide_user_options (GDM_GREETER_PANEL (session->priv->panel));
         gdm_greeter_login_window_reset (GDM_GREETER_LOGIN_WINDOW (session->priv->login_window));
 }
 
@@ -154,18 +163,16 @@ on_query_answer (GdmGreeterLoginWindow *login_window,
 }
 
 static void
-on_select_session (GdmGreeterLoginWindow *login_window,
-                   const char            *text,
-                   GdmGreeterSession     *session)
+on_select_session (GdmGreeterSession     *session,
+                   const char            *text)
 {
         gdm_greeter_client_call_select_session (session->priv->client,
                                                 text);
 }
 
 static void
-on_select_language (GdmGreeterLoginWindow *login_window,
-                    const char            *text,
-                    GdmGreeterSession     *session)
+on_select_language (GdmGreeterSession     *session,
+                    const char            *text)
 {
         gdm_greeter_client_call_select_language (session->priv->client,
                                                  text);
@@ -176,23 +183,16 @@ on_select_user (GdmGreeterLoginWindow *login_window,
                 const char            *text,
                 GdmGreeterSession     *session)
 {
+        gdm_greeter_panel_show_user_options (GDM_GREETER_PANEL (session->priv->panel));
         gdm_greeter_client_call_select_user (session->priv->client,
                                              text);
 }
 
-static void
-on_select_hostname (GdmGreeterLoginWindow *login_window,
-                    const char            *text,
-                    GdmGreeterSession     *session)
-{
-        gdm_greeter_client_call_select_hostname (session->priv->client,
-                                                 text);
-}
-
 static void
 on_cancelled (GdmGreeterLoginWindow *login_window,
               GdmGreeterSession     *session)
 {
+        gdm_greeter_panel_hide_user_options (GDM_GREETER_PANEL (session->priv->panel));
         gdm_greeter_client_call_cancel (session->priv->client);
 }
 
@@ -208,15 +208,24 @@ toggle_panel (GdmSessionManager *manager,
               gboolean           enabled,
               GdmGreeterSession *session)
 {
-#if 0
         if (enabled) {
                 session->priv->panel = gdm_greeter_panel_new ();
+
+                g_signal_connect_swapped (session->priv->panel,
+                                          "language-selected",
+                                          G_CALLBACK (on_select_language),
+                                          session);
+
+                g_signal_connect_swapped (session->priv->panel,
+                                          "session-selected",
+                                          G_CALLBACK (on_select_session),
+                                          session);
+
                 gtk_widget_show (session->priv->panel);
         } else {
                 gtk_widget_destroy (session->priv->panel);
                 session->priv->panel = NULL;
         }
-#endif
 }
 
 static void
@@ -243,22 +252,10 @@ toggle_login_window (GdmSessionManager *manager,
                                   "query-answer",
                                   G_CALLBACK (on_query_answer),
                                   session);
-                g_signal_connect (session->priv->login_window,
-                                  "session-selected",
-                                  G_CALLBACK (on_select_session),
-                                  session);
-                g_signal_connect (session->priv->login_window,
-                                  "language-selected",
-                                  G_CALLBACK (on_select_language),
-                                  session);
                 g_signal_connect (session->priv->login_window,
                                   "user-selected",
                                   G_CALLBACK (on_select_user),
                                   session);
-                g_signal_connect (session->priv->login_window,
-                                  "hostname-selected",
-                                  G_CALLBACK (on_select_hostname),
-                                  session);
                 g_signal_connect (session->priv->login_window,
                                   "cancelled",
                                   G_CALLBACK (on_cancelled),
@@ -335,11 +332,153 @@ launch_metacity (GdmGreeterSession *session)
 static void
 start_window_manager (GdmGreeterSession *session)
 {
-        if (! launch_compiz (session)) {
-                launch_metacity (session);
+        if (! launch_metacity (session)) {
+                launch_compiz (session);
         }
 }
 
+static void
+toggle_screen_reader (GdmGreeterSession *session,
+                      gboolean           enabled)
+{
+        g_debug ("GdmGreeterSession: screen reader toggled: %d", enabled);
+        gdm_session_client_set_enabled (session->priv->screen_reader_client, enabled);
+}
+
+static void
+toggle_screen_magnifier (GdmGreeterSession *session,
+                         gboolean           enabled)
+{
+        g_debug ("GdmGreeterSession: screen magnifier toggled: %d", enabled);
+        gdm_session_client_set_enabled (session->priv->screen_magnifier_client, enabled);
+}
+
+static void
+toggle_screen_keyboard (GdmGreeterSession *session,
+                        gboolean           enabled)
+{
+        g_debug ("GdmGreeterSession: screen keyboard toggled: %d", enabled);
+        gdm_session_client_set_enabled (session->priv->screen_keyboard_client, enabled);
+}
+
+static void
+on_a11y_key_changed (GConfClient       *client,
+                     guint              cnxn_id,
+                     GConfEntry        *entry,
+                     GdmGreeterSession *session)
+{
+        const char *key;
+        GConfValue *value;
+
+        key = gconf_entry_get_key (entry);
+        value = gconf_entry_get_value (entry);
+
+        if (strcmp (key, KEY_SCREEN_READER_ENABLED) == 0) {
+                if (value->type == GCONF_VALUE_BOOL) {
+                        gboolean enabled;
+
+                        enabled = gconf_value_get_bool (value);
+                        g_debug ("setting key %s = %d", key, enabled);
+                        toggle_screen_reader (session, enabled);
+                } else {
+                        g_warning ("Error retrieving configuration key '%s': Invalid type",
+                                   key);
+                }
+
+        } else if (strcmp (key, KEY_SCREEN_MAGNIFIER_ENABLED) == 0) {
+                if (value->type == GCONF_VALUE_BOOL) {
+                        gboolean enabled;
+
+                        enabled = gconf_value_get_bool (value);
+                        g_debug ("setting key %s = %d", key, enabled);
+                        toggle_screen_magnifier (session, enabled);
+                } else {
+                        g_warning ("Error retrieving configuration key '%s': Invalid type",
+                                   key);
+                }
+
+        } else if (strcmp (key, KEY_SCREEN_KEYBOARD_ENABLED) == 0) {
+                if (value->type == GCONF_VALUE_BOOL) {
+                        gboolean enabled;
+
+                        enabled = gconf_value_get_bool (value);
+                        g_debug ("setting key %s = %d", key, enabled);
+                        toggle_screen_keyboard (session, enabled);
+                } else {
+                        g_warning ("Error retrieving configuration key '%s': Invalid type",
+                                   key);
+                }
+
+        } else {
+        }
+}
+
+static void
+setup_at_tools (GdmGreeterSession *session)
+{
+        GConfClient *client;
+        gboolean     enabled;
+
+        client = gconf_client_get_default ();
+        gconf_client_add_dir (client,
+                              KEY_GDM_A11Y_DIR,
+                              GCONF_CLIENT_PRELOAD_ONELEVEL,
+                              NULL);
+        gconf_client_notify_add (client,
+                                 KEY_GDM_A11Y_DIR,
+                                 (GConfClientNotifyFunc)on_a11y_key_changed,
+                                 session,
+                                 NULL,
+                                 NULL);
+
+        session->priv->screen_keyboard_client = gdm_session_client_new ();
+        gdm_session_client_set_name (session->priv->screen_keyboard_client,
+                                     "On-screen Keyboard");
+        gdm_session_client_set_try_exec (session->priv->screen_keyboard_client,
+                                         "gok");
+        gdm_session_client_set_command (session->priv->screen_keyboard_client,
+                                        "gok");
+        enabled = gconf_client_get_bool (client, KEY_SCREEN_KEYBOARD_ENABLED, NULL);
+        gdm_session_client_set_enabled (session->priv->screen_keyboard_client,
+                                        enabled);
+
+
+        session->priv->screen_reader_client = gdm_session_client_new ();
+        gdm_session_client_set_name (session->priv->screen_reader_client,
+                                     "Screen Reader");
+        gdm_session_client_set_try_exec (session->priv->screen_reader_client,
+                                         "orca");
+        gdm_session_client_set_command (session->priv->screen_reader_client,
+                                        "orca -n");
+        enabled = gconf_client_get_bool (client, KEY_SCREEN_READER_ENABLED, NULL);
+        gdm_session_client_set_enabled (session->priv->screen_reader_client,
+                                        enabled);
+
+
+        session->priv->screen_magnifier_client = gdm_session_client_new ();
+        gdm_session_client_set_name (session->priv->screen_magnifier_client,
+                                     "Screen Magnifier");
+        gdm_session_client_set_try_exec (session->priv->screen_magnifier_client,
+                                         "magnifier");
+        gdm_session_client_set_command (session->priv->screen_magnifier_client,
+                                        "magnifier -v -m");
+        enabled = gconf_client_get_bool (client, KEY_SCREEN_MAGNIFIER_ENABLED, NULL);
+        gdm_session_client_set_enabled (session->priv->screen_magnifier_client,
+                                        enabled);
+
+        gdm_session_manager_add_client (session->priv->manager,
+                                        session->priv->screen_reader_client,
+                                        GDM_SESSION_LEVEL_LOGIN_WINDOW);
+        gdm_session_manager_add_client (session->priv->manager,
+                                        session->priv->screen_keyboard_client,
+                                        GDM_SESSION_LEVEL_LOGIN_WINDOW);
+        gdm_session_manager_add_client (session->priv->manager,
+                                        session->priv->screen_magnifier_client,
+                                        GDM_SESSION_LEVEL_LOGIN_WINDOW);
+
+        g_object_unref (client);
+}
+
 static gboolean
 start_settings_daemon (GdmGreeterSession *session)
 {
@@ -372,12 +511,7 @@ toggle_all_levels (GdmSessionManager *manager,
         if (enabled) {
                 start_settings_daemon (session);
                 start_window_manager (session);
-
-                session->priv->background = gdm_greeter_background_new ();
-                gtk_widget_show (session->priv->background);
         } else {
-                gtk_widget_destroy (session->priv->background);
-                session->priv->background = NULL;
         }
 }
 
@@ -480,6 +614,38 @@ gdm_greeter_session_class_init (GdmGreeterSessionClass *klass)
         g_type_class_add_private (klass, sizeof (GdmGreeterSessionPrivate));
 }
 
+static void
+gdm_greeter_session_event_handler(GdkEvent          *event,
+                                  GdmGreeterSession *session)
+{
+        g_assert (GDM_IS_GREETER_SESSION (session));
+
+        if (event->type == GDK_KEY_PRESS) {
+                GdkEventKey *key_event;
+
+                key_event = (GdkEventKey *) event;
+                if (session->priv->panel != NULL) {
+                        if (gtk_window_activate_key (GTK_WINDOW (session->priv->panel),
+                                                     key_event)) {
+                                gtk_window_present_with_time (GTK_WINDOW (session->priv->panel),
+                                                              key_event->time);
+                                return;
+                        }
+                }
+
+                if (session->priv->login_window != NULL) {
+                        if (gtk_window_activate_key (GTK_WINDOW (session->priv->login_window),
+                                                     ((GdkEventKey *) event))) {
+                                gtk_window_present_with_time (GTK_WINDOW (session->priv->login_window),
+                                                              key_event->time);
+                                return;
+                        }
+                }
+        }
+
+        gtk_main_do_event (event);
+}
+
 static void
 gdm_greeter_session_init (GdmGreeterSession *session)
 {
@@ -532,6 +698,14 @@ gdm_greeter_session_init (GdmGreeterSession *session)
                           G_CALLBACK (on_selected_user_changed),
                           session);
 
+        /* We want to listen for panel mnemonics even if the
+         * login window is focused, so we intercept them here.
+         */
+        gdk_event_handler_set ((GdkEventFunc) gdm_greeter_session_event_handler,
+                               session, NULL);
+
+        /* FIXME: we should really do this in settings daemon */
+        setup_at_tools (session);
 }
 
 static void
